/*
 *  Copyright 2007-2008 Hikvision Digital Technology Co., Ltd.
 *	the api for ds40xxhc ds4100 board previewing
 *	last update time: 2008-11-05
 */

#ifndef _AUDIO_PREVIEW_H_
#define _AUDIO_PREVIEW_H_

#ifdef __cplusplus
extern "C"
{
#endif

#include <stdint.h>

/* 
 * Ƶݵؽṹ 
 */
typedef struct _WAVE_STRUCT
{
	Uint8   *sound;    /* Pointer to audio data */
	Uint32  soundlen;  /* Length of audio data  */
	int     soundpos;  /* Current play position */
}WAVE_STRUCT;

/* 
 * ƵPCIԤݽṹ 
 */
typedef struct _AUDIO_CTRL
{
	sem_t       sync;     /* ƵPCIԤͬź */
                          /* յDSPƵʱSDK/sem_postź */
	pthread_t   thread;   /* ƵPCIԤݲ߳ */
	Uint8       *data;    /* SDKƵPCIԤַ, SDK/DRVȷ */
	uint32_t    datalen;  /* SDKƵPCIԤ, SDK/DSPȷ */
	uint32_t    playing;  /* 1:ƵPCIԤ, 0: رԤ */
}AUDIO_CTRL, *HAUDIO_CTRL;

SDL_AudioSpec spec;
WAVE_STRUCT	  wave;       
AUDIO_CTRL    g_aud_ctrl;

/**********************************************************
 Function:    play_sound_cb
 Description: Ƶݻص, SDLⴴִ߳
 Parameter:   unused -- δʹ [IN]
              stream -- ƵṩĻַ [IN]
              len    -- ÿβŵƵݳ [IN]
 Return:      
 *********************************************************/
void SDLCALL play_sound_cb(void *unused, Uint8 *stream, int len)
{
	Uint8 *waveptr;
	int    waveleft;

	waveptr  = wave.sound + wave.soundpos;
	waveleft = wave.soundlen - wave.soundpos;
	while (waveleft <= len)
	{
		SDL_MixAudio(stream, waveptr, waveleft, SDL_MIX_MAXVOLUME);
		stream += waveleft;
		len -= waveleft;
		waveptr = wave.sound;
		waveleft = wave.soundlen;
		wave.soundpos = 0;
	}

	SDL_MixAudio(stream, waveptr, len, SDL_MIX_MAXVOLUME);
	wave.soundpos += len;
}

/**********************************************************
 Function:    audio_preview_rtn
 Description: Ƶݲ֮߳start_routine
 Parameter:   arg --  [IN]
 Return:      
 *********************************************************/
void *audio_preview_rtn(void *arg)
{
	int started = 0, idx = 0;

    /* open sdl audio */
	if (SDL_OpenAudio(&spec, NULL) < 0)
	{
		fprintf(stderr, "Fail to open device: %s\n", SDL_GetError());
		g_aud_ctrl.playing = 0;
		g_aud_ctrl.thread = 0;

		pthread_exit(NULL);
	}

	while(1)
	{
		/* ȴƵݵź */
		if(sem_wait(&g_aud_ctrl.sync) == 0)
		{
            if(g_aud_ctrl.playing == 0)
            {
                break;
            }
			memcpy((wave.sound + idx * g_aud_ctrl.datalen), g_aud_ctrl.data, g_aud_ctrl.datalen);
			++idx;
		}
		else
		{
			usleep(100);
		}

		/* Start play after 20x40ms */
		if((idx == 20) && (started == 0))
		{
			SDL_PauseAudio(0);
			started = 1;
		}

		/* Back to buffer head */
		if(idx == (25 * 4))
		{
			idx = 0;
		}
	}

	SDL_CloseAudio();

	g_aud_ctrl.data = 0;
	wave.soundpos   = 0;
    
    free(wave.sound);

	pthread_exit(NULL);
}

int SetAudioPCIPreview_ex(int hChannelHandle, int enable)
{
	int ret;

	AUDPRE_CONFIG cfg;

    if(enable)
    {
        /* initial */
        memset(&g_aud_ctrl, 0, sizeof(g_aud_ctrl));
        memset(&wave, 0, sizeof(wave));
    }

	g_aud_ctrl.playing = enable;

	if(enable)
	{
		cfg.syncSem = &g_aud_ctrl.sync;
	}
	else
	{
		cfg.syncSem = NULL;
	}

	ret = SetAudioPCIPreview(hChannelHandle, &cfg, g_aud_ctrl.playing);
	if(ret < 0)
	{
		g_aud_ctrl.playing = 0;
		fprintf(stderr, "Audio PCI preview error!\n");
        return -1;
	}

    if(enable)
    {
        fprintf(stderr, "Audio Format : frequent = %d, samples = %d, channels = %d\n", 
                cfg.freq, cfg.samples, cfg.channels);
    }

    g_aud_ctrl.data = (Uint8 *)cfg.dataAddr;
    /* one packet per 40ms */
    g_aud_ctrl.datalen = cfg.audioSize;    

    if(enable)
    {
        /* Ŀռܴ洢4ӵƵ */
        wave.soundlen = g_aud_ctrl.datalen * (25 * 4);
        wave.sound = (Uint8 *)malloc(wave.soundlen);
        if(wave.sound == NULL)
        {
            SetAudioPCIPreview(hChannelHandle, &cfg, 0);
            g_aud_ctrl.playing = 0;
            fprintf(stderr, "Fail to allocate playing audio buffer!\n");
            return -1;
        }
        /* 
         * desired audio spec, sample rate 16k, single channel, 
         * audio data format, audio buffer size in samples 
         */
        spec.freq = cfg.freq;        
        spec.channels = cfg.channels;            
        spec.format   = AUDIO_S16LSB; 
        spec.samples  = 1024;    
	    spec.callback = play_sound_cb;
    }

    /* Create a thread to play audio */
    if(g_aud_ctrl.playing)
    {
        pthread_create(&g_aud_ctrl.thread, NULL, audio_preview_rtn, NULL);
    }
    else
    {
        /* wait audio play thread exit */
        sem_post(&g_aud_ctrl.sync);
        pthread_join(g_aud_ctrl.thread, NULL);
        g_aud_ctrl.thread = 0;
    }

    return 0;
}

int StartAudioPreview(int hChannelHandle, int enable)
{
    unsigned int boardType = 0;
    char serialNo[20]; 

    if(GetBoardInfo(hChannelHandle, &boardType, serialNo) < 0)
    {
        fprintf(stderr, "<demo_info> Channel = %d, GetBoardInfo error!\n", hChannelHandle);
        return -1;
    }

    if((boardType == DS4108HCV) || (boardType == DS4116HCV) 
	|| (boardType == DS4208HFV) || (boardType == DS4216HC) || (boardType == DS4216HFV))
    {
        if(SetAudioPCIPreview_ex(hChannelHandle, enable) < 0)
        {
            fprintf(stderr, "<demo_info> Channel = %d, SetAudioPCIPreviewEx error!\n", hChannelHandle);
            return -1;
        }
    }
    else
    {
        if(SetAudioPreview(hChannelHandle, enable) < 0)
        {
            fprintf(stderr, "<demo_info> Channel = %d, SetAudioPreview error!\n", hChannelHandle);
            return -1;
        }
    }

    return 0;
}

#ifdef __cplusplus
}
#endif

#endif

